package com.tin.tds.lower.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;

import net.sourceforge.barbecue.BarcodeException;
import net.sourceforge.barbecue.output.OutputException;

import com.tin.tds.lower.FileValidatorInterface;
import com.tin.tds.lower.bean.FileStatistics27A;
import com.tin.tds.lower.bean.PDFFileStatisticData;
import com.tin.tds.lower.bean.TDSBatchHeaderBean;
import com.tin.tds.lower.bean.TDSChallanDetailBean;
import com.tin.tds.lower.bean.TDSDeducteeDetailBean;
import com.tin.tds.lower.bean.TDSFileHeaderBean;

public class PDFGenerator {

	
	public static boolean generatePDFFile(String inputFile,String pdfFileName,String temppdfFileName,String consFile) throws BarcodeException, OutputException
	{
		int lineCount = 0;
		int noOfBatches=0;
		PDFFileStatisticData [] statisticData = null;
		
		
		FileGenerator fg = new FileGenerator();
		FileStatistics27A fstatistics27A = new FileStatistics27A();
		
		try {
			BufferedReader fileReader = new BufferedReader(new FileReader(inputFile));
					
			String lineData = null;
			int currentBatchInProcessing = 0;
			String transactionType=null;
			
			while((lineData = fileReader.readLine())!=null)
			{
				lineCount++;
				String [] recordData  = (lineData+"-").split("\\^");
				
				if(recordData[1].equals(FileValidatorInterface.FILE_HEADER_IDENTIFIER))
				{
					TDSFileHeaderBean fhData  = getFileHeaderBean(recordData);
					noOfBatches = Integer.parseInt(fhData.getNoOfBatches(),10);
					statisticData = new PDFFileStatisticData[noOfBatches];
					transactionType = fhData.getUploadType();
					fstatistics27A.setFilehash(fhData.getFileHash());
					
				}
				else if(recordData[1].equals(FileValidatorInterface.BATCH_HEADER_IDENTIFIER))
				{
					currentBatchInProcessing++;					
					TDSBatchHeaderBean bhData = getBatchHeaderBean(recordData);
					
					statisticData[currentBatchInProcessing-1] = new PDFFileStatisticData();
					
					if(transactionType!=null && transactionType.equals("C"))
					{
						statisticData[currentBatchInProcessing-1].setTransactionType("Correction Type ("+bhData.getTransactionType()+")");
					}
					else
						statisticData[currentBatchInProcessing-1].setTransactionType("Regular");
					
					fstatistics27A.setPeriod(bhData.getPeriod());
					              
					fstatistics27A.setTax_Ded_Acc_no(bhData.getTanOfDeductor());
					fstatistics27A.setPAN(bhData.getPanOfDeductor());
					fstatistics27A.setFinancial_Year(bhData.getFinancialYr());
					fstatistics27A.setAssessment_Year(bhData.getAssessmentYr());
					fstatistics27A.setForm_No(bhData.getFormNo());
					fstatistics27A.setPrevious_Prn("NA");
					fstatistics27A.setName_Deductor(bhData.getEmployerName());
					fstatistics27A.setType_Of_deductor(dedReturn(bhData.getDeductorType()));   //amit
					fstatistics27A.setBranch_Division(bhData.getEmployerBranchDiv());
					fstatistics27A.setFlat_No_Deductor(bhData.getEmployerAddr1());
					fstatistics27A.setName_Building_Deductor(bhData.getEmployerAddr2());
					fstatistics27A.setRoad_Street_Deductor(bhData.getEmployerAddr3());
					fstatistics27A.setArea_loc_Deductor(bhData.getEmployerAddr4());
					fstatistics27A.setTown_City_Deductor(bhData.getEmployerAddr5());
					fstatistics27A.setState_Deductor(stateReturn(bhData.getEmployerState())); //amit
					fstatistics27A.setPin_Code_Deductor(bhData.getEmployerPin());
					fstatistics27A.setStd_Code_Deductor(bhData.getEmployerStd());
					fstatistics27A.setTelephone_Deductor(bhData.getEmployerPhone());
					fstatistics27A.setEmail_Deductor(bhData.getEmployerEmail());
					
					fstatistics27A.setName_Person(bhData.getNameofPersonResponsilbleForSal());
					
					fstatistics27A.setFlat_No_Person(bhData.getPersonResponsilbleAddr1());
					fstatistics27A.setName_Building_Person(bhData.getPersonResponsilbleAddr2());
					fstatistics27A.setRoad_Street_Person(bhData.getPersonResponsilbleAddr3());
					fstatistics27A.setArea_loc_Person(bhData.getPersonResponsilbleAddr4());
					fstatistics27A.setTown_City_Person(bhData.getPersonResponsilbleAddr5());
					fstatistics27A.setState_Person(stateReturn(bhData.getPersonResponsilbleState())); //amit
					fstatistics27A.setPin_Code_Person(bhData.getPersonResponsilblePin());
					fstatistics27A.setStd_Code_Person(bhData.getPersonResponsilbleSTDCode());
					fstatistics27A.setTelephone_Person(bhData.getPersonResponsilbleTelePhone());
					fstatistics27A.setEmail_Person(bhData.getPersonResponsilbleEmailId1());
					fstatistics27A.setVerification_Name(bhData.getNameofPersonResponsilbleForSal());
					fstatistics27A.setResp_Prsn_Designation(bhData.getDesignationofPersonResponsilbleForSal());
					fstatistics27A.setVerification_Palce(bhData.getEmployerAddr5());
					fstatistics27A.setPanOfResponsiblePerson(bhData.getRegistrationId()); //Added for FVU 2.142
					DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy",Locale.ENGLISH);
					Calendar cal = Calendar.getInstance();
					fstatistics27A.setVerification_Date(dateFormat.format(cal.getTime()).toString());
				}
				else if(recordData[1].equals(FileValidatorInterface.CHALLAN_HEADER_IDENTIFIER))
				{
					TDSChallanDetailBean cdData = getChallanDetailBean(recordData);
					
					statisticData[currentBatchInProcessing-1].addnoOfDeductee(cdData.getCountOfDeducteeDetail());
					statisticData[currentBatchInProcessing-1].addtaxDeposited(cdData.getTotalOfDepositAmountAsPerChallan());
				}
				else if(recordData[1].equals(FileValidatorInterface.DEDUCTEE_HEADER_IDENTIFIER))
				{
					TDSDeducteeDetailBean ddData = getDeducteeDetailBean(recordData);
					
					statisticData[currentBatchInProcessing-1].addtaxDeducted(ddData.getTotalIncomeTaxDeductedAtSource());
					statisticData[currentBatchInProcessing-1].addamtPaid(ddData.getAmountOfPayment());

				}
				
			}
			
			fileReader.close();
			if(consFile!=null && !consFile.equals(""))
				{
				
					BufferedReader consofileReader=new BufferedReader(new FileReader(consFile));
					while((lineData = consofileReader.readLine())!=null)
					
					{
						lineCount++;
						String [] recordData  = (lineData+"-").split("\\^");
						if(recordData[1].equals(FileValidatorInterface.BATCH_HEADER_IDENTIFIER))
						{
										
							TDSBatchHeaderBean bhData = getBatchHeaderBean(recordData);
													
							fstatistics27A.setPeriod(bhData.getPeriod());
							              
							fstatistics27A.setTax_Ded_Acc_no(bhData.getTanOfDeductor());
							fstatistics27A.setPAN(bhData.getPanOfDeductor());
							fstatistics27A.setFinancial_Year(bhData.getFinancialYr());
							fstatistics27A.setAssessment_Year(bhData.getAssessmentYr());
							fstatistics27A.setForm_No(bhData.getFormNo());
							if(bhData.getPreviousRrrNo()!=null && !bhData.getPreviousRrrNo().equals(""))
								fstatistics27A.setPrevious_Prn(bhData.getPreviousRrrNo());
							else
								fstatistics27A.setPrevious_Prn("NA");
							fstatistics27A.setName_Deductor(bhData.getEmployerName());
							fstatistics27A.setType_Of_deductor(dedReturn(bhData.getDeductorType())); //amit
							fstatistics27A.setBranch_Division(bhData.getEmployerBranchDiv());
							fstatistics27A.setFlat_No_Deductor(bhData.getEmployerAddr1());
							fstatistics27A.setName_Building_Deductor(bhData.getEmployerAddr2());
							fstatistics27A.setRoad_Street_Deductor(bhData.getEmployerAddr3());
							fstatistics27A.setArea_loc_Deductor(bhData.getEmployerAddr4());
							fstatistics27A.setTown_City_Deductor(bhData.getEmployerAddr5());
							fstatistics27A.setState_Deductor(stateReturn(bhData.getEmployerState())); //amit
							fstatistics27A.setPin_Code_Deductor(bhData.getEmployerPin());
							fstatistics27A.setStd_Code_Deductor(bhData.getEmployerStd());
							fstatistics27A.setTelephone_Deductor(bhData.getEmployerPhone());
							fstatistics27A.setEmail_Deductor(bhData.getEmployerEmail());
							
							fstatistics27A.setName_Person(bhData.getNameofPersonResponsilbleForSal());
							
							fstatistics27A.setFlat_No_Person(bhData.getPersonResponsilbleAddr1());
							fstatistics27A.setName_Building_Person(bhData.getPersonResponsilbleAddr2());
							fstatistics27A.setRoad_Street_Person(bhData.getPersonResponsilbleAddr3());
							fstatistics27A.setArea_loc_Person(bhData.getPersonResponsilbleAddr4());
							fstatistics27A.setTown_City_Person(bhData.getPersonResponsilbleAddr5());
							fstatistics27A.setState_Person(stateReturn(bhData.getPersonResponsilbleState())); //amit
							fstatistics27A.setPin_Code_Person(bhData.getPersonResponsilblePin());
							fstatistics27A.setStd_Code_Person(bhData.getPersonResponsilbleSTDCode());
							fstatistics27A.setTelephone_Person(bhData.getPersonResponsilbleTelePhone());
							fstatistics27A.setEmail_Person(bhData.getPersonResponsilbleEmailId1());
							fstatistics27A.setVerification_Name(bhData.getNameofPersonResponsilbleForSal());
							fstatistics27A.setResp_Prsn_Designation(bhData.getDesignationofPersonResponsilbleForSal());
							fstatistics27A.setVerification_Palce(bhData.getEmployerAddr5());
							fstatistics27A.setPanOfResponsiblePerson(bhData.getRegistrationId()); //Added for FVU 2.142
							DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy",Locale.ENGLISH);
							Calendar cal = Calendar.getInstance();
							fstatistics27A.setVerification_Date(dateFormat.format(cal.getTime()).toString());
							break;
						}
					}
					consofileReader.close();
				}
			
			String fvuPath=temppdfFileName.substring(0,temppdfFileName.lastIndexOf('\\')+1);
			String fvuFileName=inputFile.substring(inputFile.lastIndexOf('\\')+1, inputFile.lastIndexOf("."));
			File fvuFile = new File(fvuPath+fvuFileName+".fvu");
			lineCount=0;
			if(fvuFile.exists())
			{
			
				BufferedReader fvufileReader=new BufferedReader(new FileReader(fvuFile));
				while((lineData = fvufileReader.readLine())!=null && noOfBatches>=lineCount)
				
				{
					lineCount++;
					String [] recordData  = (lineData+"-").split("\\^");
					if(recordData[1].equals(FileValidatorInterface.FILE_HEADER_IDENTIFIER))
					{
						TDSFileHeaderBean fhData  = getFileHeaderBean(recordData);		
						fstatistics27A.setFilehash(fhData.getFileHash());
					}
					if(recordData[1].equals(FileValidatorInterface.BATCH_HEADER_IDENTIFIER))
					{
						TDSBatchHeaderBean bhData = getBatchHeaderBean(recordData);
						if(bhData.getTransactionType().equals("C2") || bhData.getTransactionType().equals("C3") || bhData.getTransactionType().equals("C4") || bhData.getTransactionType().equals("C5") || bhData.getTransactionType().equals("C9"))
						{
							fstatistics27A.setPanOfResponsiblePerson(bhData.getRegistrationId()); //Added for FVU 4.6
						}
						if(bhData.getTransactionType().equals("C1") || bhData.getBatchUpdationIndicator().equals("1"))
						{
						
							fstatistics27A.setPeriod(bhData.getPeriod());
							             
							fstatistics27A.setTax_Ded_Acc_no(bhData.getTanOfDeductor());
							fstatistics27A.setPAN(bhData.getPanOfDeductor());
							fstatistics27A.setFinancial_Year(bhData.getFinancialYr());
							fstatistics27A.setAssessment_Year(bhData.getAssessmentYr());
							fstatistics27A.setForm_No(bhData.getFormNo());
							if(bhData.getPreviousRrrNo()!=null && !bhData.getPreviousRrrNo().equals(""))
								fstatistics27A.setPrevious_Prn(bhData.getPreviousRrrNo());
							else
								fstatistics27A.setPrevious_Prn("NA");
							fstatistics27A.setName_Deductor(bhData.getEmployerName());
							fstatistics27A.setType_Of_deductor(dedReturn(bhData.getDeductorType())); //amitgethere
							fstatistics27A.setBranch_Division(bhData.getEmployerBranchDiv());
							fstatistics27A.setFlat_No_Deductor(bhData.getEmployerAddr1());
							fstatistics27A.setName_Building_Deductor(bhData.getEmployerAddr2());
							fstatistics27A.setRoad_Street_Deductor(bhData.getEmployerAddr3());
							fstatistics27A.setArea_loc_Deductor(bhData.getEmployerAddr4());
							fstatistics27A.setTown_City_Deductor(bhData.getEmployerAddr5());
							fstatistics27A.setState_Deductor(stateReturn(bhData.getEmployerState()));  //amit2
							fstatistics27A.setPin_Code_Deductor(bhData.getEmployerPin());
							fstatistics27A.setStd_Code_Deductor(bhData.getEmployerStd());
							fstatistics27A.setTelephone_Deductor(bhData.getEmployerPhone());
							fstatistics27A.setEmail_Deductor(bhData.getEmployerEmail());
							
							fstatistics27A.setName_Person(bhData.getNameofPersonResponsilbleForSal());
							
							fstatistics27A.setFlat_No_Person(bhData.getPersonResponsilbleAddr1());
							fstatistics27A.setName_Building_Person(bhData.getPersonResponsilbleAddr2());
							fstatistics27A.setRoad_Street_Person(bhData.getPersonResponsilbleAddr3());
							fstatistics27A.setArea_loc_Person(bhData.getPersonResponsilbleAddr4());
							fstatistics27A.setTown_City_Person(bhData.getPersonResponsilbleAddr5());
							fstatistics27A.setState_Person(stateReturn(bhData.getPersonResponsilbleState())); //amit 3
							fstatistics27A.setPin_Code_Person(bhData.getPersonResponsilblePin());
							fstatistics27A.setStd_Code_Person(bhData.getPersonResponsilbleSTDCode());
							fstatistics27A.setTelephone_Person(bhData.getPersonResponsilbleTelePhone());
							fstatistics27A.setEmail_Person(bhData.getPersonResponsilbleEmailId1());
							fstatistics27A.setVerification_Name(bhData.getNameofPersonResponsilbleForSal());
							fstatistics27A.setResp_Prsn_Designation(bhData.getDesignationofPersonResponsilbleForSal());
							fstatistics27A.setVerification_Palce(bhData.getEmployerAddr5());
							fstatistics27A.setPanOfResponsiblePerson(bhData.getRegistrationId()); //Added for FVU 2.142
							DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy",Locale.ENGLISH);
							Calendar cal = Calendar.getInstance();
							fstatistics27A.setVerification_Date(dateFormat.format(cal.getTime()).toString());
							break;
						}
					}
				}
				fvufileReader.close();
				}
			
			boolean generated;
			generated=fg.generateForm27A(temppdfFileName,pdfFileName,fstatistics27A,statisticData);
			return generated;
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return true;
	}
	
	
	//Added By amit
	private static String dedReturn(String ded_type)
	{
		if(ded_type!=null)
			ded_type=ded_type.trim();
		
		if(ded_type.equals("A"))
			 ded_type="Central Government";
		 else if( ded_type.equals("S"))
			 ded_type="State Government";
		 else if(ded_type.equals("D"))
			 ded_type="Statutory body (Central Govt.)";
		 else if( ded_type.equals("E"))
			 ded_type="Statutory body (State Govt.)";
		 else if( ded_type.equals("G"))
			 ded_type="Autonomous body (Central Govt.)";
		 else if( ded_type.equals("H"))
			 ded_type="Autonomous body (State Govt.)";
		 else if( ded_type.equals("L"))
			 ded_type="Local Authority (Central Govt.)";
		 else if( ded_type.equals("N"))
			 ded_type="Local Authority (State Govt.)";
		 else if( ded_type.equals("K"))
			 ded_type="Company";
		 else if( ded_type.equals("M"))
			 ded_type="Branch / Division of Company";
		 else if( ded_type.equals("P"))
			 ded_type="Association of Person (AOP)";
		 else if( ded_type.equals("T"))
			 ded_type="Association of Person (Trust)";
		 else if( ded_type.equals("J"))
			 ded_type="Artificial Juridical Person";
		 else if( ded_type.equals("B"))
			 ded_type="Body of Individuals";
		 else if( ded_type.equals("Q"))
			 ded_type="Individual/HUF";
		 else if( ded_type.equals("F"))
			 ded_type="Firm";
		
		
		return ded_type;
	}
	
	
	
	
	
	private static String stateReturn(String emp_state) 
	 
	  {	  
		if(emp_state!=null)
			emp_state=emp_state.trim();
		if(emp_state.equals("1") || emp_state.equals("01"))
		  emp_state="ANDAMAN AND NICOBAR ISLANDS";
	  else if(emp_state.equals("2") || emp_state.equals("02"))
		  emp_state="ANDHRA PRADESH";
	  else if(emp_state.equals("3") || emp_state.equals("03"))
		  emp_state="ARUNACHAL PRADESH";
	  else if(emp_state.equals("4") || emp_state.equals("04"))
		  emp_state="ASSAM";
	  else if(emp_state.equals("5") || emp_state.equals("05"))
		  emp_state="BIHAR";
	  else if(emp_state.equals("6") || emp_state.equals("06"))
		  emp_state="CHANDIGARH";
	  else if(emp_state.equals("7") || emp_state.equals("07"))
		//emp_state="DADRA & NAGAR HAVELI"; // FVU 7.1 req update (7.0 CR related) 10 Mar 2021 - Vinita
		  emp_state="DADRA NAGAR HAVELI & DAMAN DIU";
	  else if(emp_state.equals("8") || emp_state.equals("08"))
		  emp_state="DAMAN & DIU";
	  else if(emp_state.equals("9") || emp_state.equals("09"))
		  emp_state="DELHI";
	  else if(emp_state.equals("10"))
		  emp_state="GOA";
	  else if(emp_state.equals("11"))
		  emp_state="GUJARAT";
	  else if(emp_state.equals("12"))
		  emp_state="HARYANA";
	  else if(emp_state.equals("13"))
		  emp_state="HIMACHAL PRADESH";
	  else if(emp_state.equals("14"))
		  emp_state="JAMMU & KASHMIR";
	  else if(emp_state.equals("15"))
		  emp_state="KARNATAKA";
	  else if(emp_state.equals("16"))
		  emp_state="KERALA";
	  else if(emp_state.equals("17"))
		  emp_state="LAKSHWADEEP";
	  else if(emp_state.equals("18"))
		  emp_state="MADHYA PRADESH";
	  else if(emp_state.equals("19"))
		  emp_state="MAHARASHTRA";
	  else if(emp_state.equals("20"))
		  emp_state="MANIPUR";
	  else if(emp_state.equals("21"))
		  emp_state="MEGHALAYA";
	  else if(emp_state.equals("22"))
		  emp_state="MIZORAM";
	  else if(emp_state.equals("23"))
		  emp_state="NAGALAND";
	  else if(emp_state.equals("24"))
		  emp_state="ODISHA";  // Modified for FVU 2.146
	  else if(emp_state.equals("25"))
		  emp_state="PONDICHERRY";
	  else if(emp_state.equals("26"))
		  emp_state="PUNJAB";
	  else if(emp_state.equals("27"))
		  emp_state="RAJASTHAN";
	  else if(emp_state.equals("28"))
		  emp_state="SIKKIM";
	  else if(emp_state.equals("29"))
		  emp_state="TAMILNADU";
	  else if(emp_state.equals("30"))
		  emp_state="TRIPURA";
	  else if(emp_state.equals("31"))
		  emp_state="UTTAR PRADESH";
	  else if(emp_state.equals("32"))
		  emp_state="WEST BENGAL";
	  else if(emp_state.equals("33"))
		  emp_state="CHHATISHGARH";
	  else if(emp_state.equals("34"))
		  //emp_state="UTTARANCHAL";
		  emp_state = "UTTARAKHAND";
	  else if(emp_state.equals("35"))
		  emp_state="JHARKHAND";
	  else if(emp_state.equals("36"))
		  emp_state="TELANGANA";
	  else if(emp_state.equals("37"))
		  emp_state="LADAKH";
	  
		 return emp_state;
	 
	 }
	//End By amit
	
	
	private static TDSFileHeaderBean getFileHeaderBean(String [] fhFields)
	{
		
		TDSFileHeaderBean fhRecord = new TDSFileHeaderBean();
		if(fhFields.length==18) //Above 2.126
		{
			fhRecord.setLineNo(fhFields[0]);
			fhRecord.setRecordType(fhFields[1]);
			fhRecord.setFileType(fhFields[2]);
			fhRecord.setUploadType(fhFields[3]);
			fhRecord.setFileDate(fhFields[4]);
			fhRecord.setFileSeq(fhFields[5]);
			fhRecord.setUploaderType(fhFields[6]);
			fhRecord.setTanTfcId(fhFields[7]);
			fhRecord.setNoOfBatches(fhFields[8]);
			fhRecord.setRpuName(fhFields[9]);
			fhRecord.setRecordHash(fhFields[10]);
			fhRecord.setFvuVersion(fhFields[11]);
			fhRecord.setFileHash(fhFields[12]);
			fhRecord.setSamVersion(fhFields[13]);
			fhRecord.setSamHash(fhFields[14]);
			fhRecord.setScmVersion(fhFields[15]);
			fhRecord.setScmHash(fhFields[16]);
			fhRecord.setConsHash(fhFields[17].substring(0, fhFields[17].length()-1));//Removing -
		}
		else if(fhFields.length==16)//2.126 and below
		{
			fhRecord.setLineNo(fhFields[0]);
			fhRecord.setRecordType(fhFields[1]);
			fhRecord.setFileType(fhFields[2]);
			fhRecord.setUploadType(fhFields[3]);
			fhRecord.setFileDate(fhFields[4]);
			fhRecord.setFileSeq(fhFields[5]);
			fhRecord.setUploaderType(fhFields[6]);
			fhRecord.setTanTfcId(fhFields[7]);
			fhRecord.setNoOfBatches(fhFields[8]);
			fhRecord.setRecordHash(fhFields[9]);
			fhRecord.setFvuVersion(fhFields[10]);
			fhRecord.setFileHash(fhFields[11]);
			fhRecord.setSamVersion(fhFields[12]);
			fhRecord.setSamHash(fhFields[13]);
			fhRecord.setScmVersion(fhFields[14]);
			fhRecord.setScmHash(fhFields[15].substring(0, fhFields[16].length()-1)); //Removing -
		}
		
		
		return fhRecord;
	}
	
	private static TDSBatchHeaderBean getBatchHeaderBean(String [] bhFields)
	{
		TDSBatchHeaderBean batchRecords = new TDSBatchHeaderBean();
		
		//if(bhFields.length==69) //2.126 above
		if(bhFields.length==70) //2.152 & above
		{
			batchRecords.setLineNo(bhFields[0]);
			batchRecords.setRecordType(bhFields[1]);
			batchRecords.setBatchNo(bhFields[2]);
			batchRecords.setChallanCount(bhFields[3]);
			batchRecords.setFormNo(bhFields[4]);
			
			batchRecords.setTransactionType(bhFields[5]);
			batchRecords.setBatchUpdationIndicator(bhFields[6]);
			batchRecords.setOriginalRrrNo(bhFields[7]);
			batchRecords.setPreviousRrrNo(bhFields[8]);
			batchRecords.setRrrNo(bhFields[9]);
			batchRecords.setRrrDate(bhFields[10]);
			batchRecords.setLastTanOfDeductor(bhFields[11]);
			batchRecords.setTanOfDeductor(bhFields[12]);
			batchRecords.setReceiptNumber(bhFields[13]);
			batchRecords.setPanOfDeductor(bhFields[14]);
			batchRecords.setAssessmentYr(bhFields[15]);
			batchRecords.setFinancialYr(bhFields[16]);
			batchRecords.setPeriod(bhFields[17]);
			batchRecords.setEmployerName(bhFields[18]);
			batchRecords.setEmployerBranchDiv(bhFields[19]);
			batchRecords.setEmployerAddr1(bhFields[20]);
			batchRecords.setEmployerAddr2(bhFields[21]);
			batchRecords.setEmployerAddr3(bhFields[22]);
			batchRecords.setEmployerAddr4(bhFields[23]);
			batchRecords.setEmployerAddr5(bhFields[24]);
			batchRecords.setEmployerState(bhFields[25]);
			batchRecords.setEmployerPin(bhFields[26]);
			batchRecords.setEmployerEmail(bhFields[27]);
			batchRecords.setEmployerStd(bhFields[28]);
			batchRecords.setEmployerPhone(bhFields[29]);
			batchRecords.setEmployerAddrChange(bhFields[30]);
			batchRecords.setDeductorType(bhFields[31]);
			batchRecords.setNameofPersonResponsilbleForSal(bhFields[32]);
			batchRecords.setDesignationofPersonResponsilbleForSal(bhFields[33]);
			batchRecords.setPersonResponsilbleAddr1(bhFields[34]);
			batchRecords.setPersonResponsilbleAddr2(bhFields[35]);
			batchRecords.setPersonResponsilbleAddr3(bhFields[36]);
			batchRecords.setPersonResponsilbleAddr4(bhFields[37]);
			batchRecords.setPersonResponsilbleAddr5(bhFields[38]);
			batchRecords.setPersonResponsilbleState(bhFields[39]);
			batchRecords.setPersonResponsilblePin(bhFields[40]);
			batchRecords.setPersonResponsilbleEmailId1(bhFields[41]);
			batchRecords.setMobileNumber(bhFields[42]);
			batchRecords.setPersonResponsilbleSTDCode(bhFields[43]);
			batchRecords.setPersonResponsilbleTelePhone(bhFields[44]);
			batchRecords.setPersonResponsilbleAddrChange(bhFields[45]);
			batchRecords.setGrossTdsTotalAsPerChallan(bhFields[46]);
			batchRecords.setUnMatchedChalanCnt(bhFields[47]);
			batchRecords.setCountOfSalaryDetailRec(bhFields[48]);
			batchRecords.setGrossTotalIncomeSd(bhFields[49]);
			batchRecords.setApprovalTaken(bhFields[50]);
			batchRecords.setApprovalNo(bhFields[51]);
			batchRecords.setLastDeductorType(bhFields[52]);
			batchRecords.setStateName(bhFields[53]);
			batchRecords.setPaoCode(bhFields[54]);
			batchRecords.setDdoCode(bhFields[55]);
			batchRecords.setMinistryName(bhFields[56]);
			batchRecords.setMinistryNameOther(bhFields[57]);
			batchRecords.setRegistrationId(bhFields[58]);
			batchRecords.setPaoRegistrationNo(bhFields[59]);
			batchRecords.setDdoRegistrationNo(bhFields[60]);
			batchRecords.setBatchHash(bhFields[61]);
		}	
		if(bhFields.length==69) //2.152 & above
		{
			batchRecords.setLineNo(bhFields[0]);
			batchRecords.setRecordType(bhFields[1]);
			batchRecords.setBatchNo(bhFields[2]);
			batchRecords.setChallanCount(bhFields[3]);
			batchRecords.setFormNo(bhFields[4]);
			
			batchRecords.setTransactionType(bhFields[5]);
			batchRecords.setBatchUpdationIndicator(bhFields[6]);
			batchRecords.setOriginalRrrNo(bhFields[7]);
			batchRecords.setPreviousRrrNo(bhFields[8]);
			batchRecords.setRrrNo(bhFields[9]);
			batchRecords.setRrrDate(bhFields[10]);
			batchRecords.setLastTanOfDeductor(bhFields[11]);
			batchRecords.setTanOfDeductor(bhFields[12]);
			batchRecords.setReceiptNumber(bhFields[13]);
			batchRecords.setPanOfDeductor(bhFields[14]);
			batchRecords.setAssessmentYr(bhFields[15]);
			batchRecords.setFinancialYr(bhFields[16]);
			batchRecords.setPeriod(bhFields[17]);
			batchRecords.setEmployerName(bhFields[18]);
			batchRecords.setEmployerBranchDiv(bhFields[19]);
			batchRecords.setEmployerAddr1(bhFields[20]);
			batchRecords.setEmployerAddr2(bhFields[21]);
			batchRecords.setEmployerAddr3(bhFields[22]);
			batchRecords.setEmployerAddr4(bhFields[23]);
			batchRecords.setEmployerAddr5(bhFields[24]);
			batchRecords.setEmployerState(bhFields[25]);
			batchRecords.setEmployerPin(bhFields[26]);
			batchRecords.setEmployerEmail(bhFields[27]);
			batchRecords.setEmployerStd(bhFields[28]);
			batchRecords.setEmployerPhone(bhFields[29]);
			batchRecords.setEmployerAddrChange(bhFields[30]);
			batchRecords.setDeductorType(bhFields[31]);
			batchRecords.setNameofPersonResponsilbleForSal(bhFields[32]);
			batchRecords.setDesignationofPersonResponsilbleForSal(bhFields[33]);
			batchRecords.setPersonResponsilbleAddr1(bhFields[34]);
			batchRecords.setPersonResponsilbleAddr2(bhFields[35]);
			batchRecords.setPersonResponsilbleAddr3(bhFields[36]);
			batchRecords.setPersonResponsilbleAddr4(bhFields[37]);
			batchRecords.setPersonResponsilbleAddr5(bhFields[38]);
			batchRecords.setPersonResponsilbleState(bhFields[39]);
			batchRecords.setPersonResponsilblePin(bhFields[40]);
			batchRecords.setPersonResponsilbleEmailId1(bhFields[41]);
			batchRecords.setMobileNumber(bhFields[42]);
			batchRecords.setPersonResponsilbleSTDCode(bhFields[43]);
			batchRecords.setPersonResponsilbleTelePhone(bhFields[44]);
			batchRecords.setPersonResponsilbleAddrChange(bhFields[45]);
			batchRecords.setGrossTdsTotalAsPerChallan(bhFields[46]);
			batchRecords.setUnMatchedChalanCnt(bhFields[47]);
			batchRecords.setCountOfSalaryDetailRec(bhFields[48]);
			batchRecords.setGrossTotalIncomeSd(bhFields[49]);
			batchRecords.setApprovalTaken(bhFields[50]);
			batchRecords.setApprovalNo(bhFields[51]);
			batchRecords.setLastDeductorType(bhFields[52]);
			batchRecords.setStateName(bhFields[53]);
			batchRecords.setPaoCode(bhFields[54]);
			batchRecords.setDdoCode(bhFields[55]);
			batchRecords.setMinistryName(bhFields[56]);
			batchRecords.setMinistryNameOther(bhFields[57]);
			batchRecords.setRegistrationId(bhFields[58]);
			batchRecords.setPaoRegistrationNo(bhFields[59]);
			batchRecords.setDdoRegistrationNo(bhFields[60]);
			batchRecords.setBatchHash(bhFields[61]);
		}
		else if(bhFields.length==68)  //Changed for FVU 3.8
		{
			batchRecords.setLineNo(bhFields[0]);
			batchRecords.setRecordType(bhFields[1]);
			batchRecords.setBatchNo(bhFields[2]);
			batchRecords.setChallanCount(bhFields[3]);
			batchRecords.setFormNo(bhFields[4]);
			batchRecords.setTransactionType(bhFields[5]);
			batchRecords.setBatchUpdationIndicator(bhFields[6]);
			batchRecords.setOriginalRrrNo(bhFields[7]);
			batchRecords.setPreviousRrrNo(bhFields[8]);
			batchRecords.setRrrNo(bhFields[9]);
			batchRecords.setRrrDate(bhFields[10]);
			batchRecords.setLastTanOfDeductor(bhFields[11]);
			batchRecords.setTanOfDeductor(bhFields[12]);
			batchRecords.setReceiptNumber(bhFields[13]);
			batchRecords.setPanOfDeductor(bhFields[14]);
			batchRecords.setAssessmentYr(bhFields[15]);
			batchRecords.setFinancialYr(bhFields[16]);
			batchRecords.setPeriod(bhFields[17]);
			batchRecords.setEmployerName(bhFields[18]);
			batchRecords.setEmployerBranchDiv(bhFields[19]);
			batchRecords.setEmployerAddr1(bhFields[20]);
			batchRecords.setEmployerAddr2(bhFields[21]);
			batchRecords.setEmployerAddr3(bhFields[22]);
			batchRecords.setEmployerAddr4(bhFields[23]);
			batchRecords.setEmployerAddr5(bhFields[24]);
			batchRecords.setEmployerState(bhFields[25]);
			batchRecords.setEmployerPin(bhFields[26]);
			batchRecords.setEmployerEmail(bhFields[27]);
			batchRecords.setEmployerStd(bhFields[28]);
			batchRecords.setEmployerPhone(bhFields[29]);
			batchRecords.setEmployerAddrChange(bhFields[30]);
			batchRecords.setDeductorType(bhFields[31]);
			batchRecords.setNameofPersonResponsilbleForSal(bhFields[32]);
			batchRecords.setDesignationofPersonResponsilbleForSal(bhFields[33]);
			batchRecords.setPersonResponsilbleAddr1(bhFields[34]);
			batchRecords.setPersonResponsilbleAddr2(bhFields[35]);
			batchRecords.setPersonResponsilbleAddr3(bhFields[36]);
			batchRecords.setPersonResponsilbleAddr4(bhFields[37]);
			batchRecords.setPersonResponsilbleAddr5(bhFields[38]);
			batchRecords.setPersonResponsilbleState(bhFields[39]);
			batchRecords.setPersonResponsilblePin(bhFields[40]);
			batchRecords.setPersonResponsilbleEmailId1(bhFields[41]);
			batchRecords.setMobileNumber(bhFields[42]);
			batchRecords.setPersonResponsilbleSTDCode(bhFields[43]);
			batchRecords.setPersonResponsilbleTelePhone(bhFields[44]);
			batchRecords.setPersonResponsilbleAddrChange(bhFields[45]);
			batchRecords.setGrossTdsTotalAsPerChallan(bhFields[46]);
			batchRecords.setUnMatchedChalanCnt(bhFields[47]);
			batchRecords.setCountOfSalaryDetailRec(bhFields[48]);
			batchRecords.setGrossTotalIncomeSd(bhFields[49]);
			batchRecords.setApprovalTaken(bhFields[50]);
			batchRecords.setApprovalNo(bhFields[51]);
			
			//Added for FVU 3.8 by Shalin
			
			batchRecords.setLastDeductorType(bhFields[52]);
			batchRecords.setStateName(bhFields[53]);
			batchRecords.setPaoCode(bhFields[54]);
			batchRecords.setDdoCode(bhFields[55]);
			batchRecords.setMinistryName(bhFields[56]);
			batchRecords.setMinistryNameOther(bhFields[57]);
			batchRecords.setRegistrationId(bhFields[58]);
			batchRecords.setPaoRegistrationNo(bhFields[59]);
			batchRecords.setDdoRegistrationNo(bhFields[60]);
			batchRecords.setEmployerSTDAlt(bhFields[61]);
			batchRecords.setEmployerPhoneAlt(bhFields[62]);
			batchRecords.setEmployerEmailAlt(bhFields[63]);
			batchRecords.setPersonResponsilbleSTDCodeAlt(bhFields[64]);
			batchRecords.setPersonResponsilbleTelePhoneAlt(bhFields[65]);
			batchRecords.setPersonResponsilbleEmailIdAlt(bhFields[66]);
			batchRecords.setAIN(bhFields[67]);
					
			
			batchRecords.setBatchHash(bhFields[68]);
		}
		
		
		return batchRecords;
	}
	
	private static TDSChallanDetailBean getChallanDetailBean(String cdBean[])
	{
		TDSChallanDetailBean cdRecord = new TDSChallanDetailBean();
		cdRecord.setLineNo(cdBean[0]);
		cdRecord.setRecType(cdBean[1]);
		cdRecord.setChBatchNo(cdBean[2]);
		cdRecord.setChallanDetailRecordNo(cdBean[3]);
		cdRecord.setCountOfDeducteeDetail(cdBean[4]);
		cdRecord.setNillChallanIndicator(cdBean[5]);
		cdRecord.setChallanUpdationIndicator(cdBean[6]);
		cdRecord.setPeriod(cdBean[7]);
		cdRecord.setStmtFromDate(cdBean[8]);
		cdRecord.setStmtToDate(cdBean[9]);
		cdRecord.setLastBankChallanNo(cdBean[10]);
		cdRecord.setBankChallanNo(cdBean[11]);
		cdRecord.setLastTransferVoucherNo(cdBean[12]);
		cdRecord.setTransferVoucherNo(cdBean[13]);
		cdRecord.setLastBankBranchCode(cdBean[14]);
		cdRecord.setBankBranchCode(cdBean[15]);
		cdRecord.setLastDateOfBankChallanNo(cdBean[16]);
		cdRecord.setDateOfBankChallanNo(cdBean[17]);
		cdRecord.setMajorHeadCode(cdBean[18]);
		cdRecord.setMinorHeadCode(cdBean[19]);
		cdRecord.setSection(cdBean[20]);
		cdRecord.setOltasIncomeTax(cdBean[21]);
		cdRecord.setOltasSurcharge(cdBean[22]);
		cdRecord.setOltasCess(cdBean[23]);
		cdRecord.setOltasInterest(cdBean[24]);
		cdRecord.setOltasOthers(cdBean[25]);
		cdRecord.setTotalOfDepositAmountAsPerChallan(cdBean[26]);
		cdRecord.setLastTotalOfDepositAmountAsPerChallan(cdBean[27]);
		cdRecord.setTotalTaxDepositedAsPerDeducteeAnex(cdBean[28]);
		cdRecord.setTdsIncomeTaxC(cdBean[29]);
		cdRecord.setTdsSurchargeC(cdBean[30]);
		cdRecord.setTdsCessC(cdBean[31]);
		cdRecord.setSumTotalIncTaxDedAtSource(cdBean[32]);
		cdRecord.setTdsInterest(cdBean[33]);
		cdRecord.setTdsOthers(cdBean[34]);
		cdRecord.setChequeDDNo(cdBean[35]);
		cdRecord.setBookCash(cdBean[36]);
		cdRecord.setRemark(cdBean[37]);
		
		//Added for FVU 3.8 by Shalin
		cdRecord.setLateFee(cdBean[38]);
		cdRecord.setMinorHeadCodeChaln(cdBean[39]);
		
		cdRecord.setChallanHash(cdBean[40]);
		
		return cdRecord;
	}

	private static TDSDeducteeDetailBean getDeducteeDetailBean(String ddBean[])
	{
		TDSDeducteeDetailBean ddRecord = new TDSDeducteeDetailBean();
		ddRecord.setLineNo(ddBean[0]);
		ddRecord.setRecType(ddBean[1]);
		ddRecord.setDdBatchNo(ddBean[2]);
		ddRecord.setChallanRecordNo(ddBean[3]);
		ddRecord.setDeducteeDetailRecNo(ddBean[4]);
		ddRecord.setMode(ddBean[5]);
		ddRecord.setEmployeeSerialNo(ddBean[6]);
		ddRecord.setDeducteeCode(ddBean[7]);
		ddRecord.setLastEmployeePan(ddBean[8]);
		ddRecord.setEmployeePan(ddBean[9]);
		ddRecord.setLastEmployeePanRefNo(ddBean[10]);
		ddRecord.setEmployeePanRefNo(ddBean[11]);
		ddRecord.setEmployeeName(ddBean[12]);
		ddRecord.setTdsIncomeTaxDD(ddBean[13]);
		ddRecord.setTdsSurchargeDD(ddBean[14]);
		ddRecord.setTdsCessDD(ddBean[15]);
		ddRecord.setTotalIncomeTaxDeductedAtSource(ddBean[16]);
		ddRecord.setLastTotalIncomeTaxDeductedAtSource(ddBean[17]);
		ddRecord.setTotalTaxDeposited(ddBean[18]);
		ddRecord.setLastTotalTaxDeposited(ddBean[19]);
		ddRecord.setTotalValueofPurchase(ddBean[20]);
		ddRecord.setAmountOfPayment(ddBean[21]);
		ddRecord.setDateOnWhichAmountPaid(ddBean[22]);
		ddRecord.setDateOnWhichTaxDeducted(ddBean[23]);
		ddRecord.setDateOfDeposit(ddBean[24]);
		ddRecord.setRateAtWhichTaxDeducted(ddBean[25]);
		ddRecord.setGrossingUpIndicator(ddBean[26]);
		ddRecord.setBookCashEntry(ddBean[27]);
		ddRecord.setDateOfFurnishingTaxDeductionCertificate(ddBean[28]);
		ddRecord.setRemark1(ddBean[29]);
		ddRecord.setRemark2(ddBean[30]);
		ddRecord.setRemark3(ddBean[31]);
		
		//Added for FVU 3.8 by Shalin
		
		ddRecord.setSectionCode(ddBean[32]);
		ddRecord.setCertNumAo(ddBean[33]);
		ddRecord.setTdsRate(ddBean[34]);
		ddRecord.setNatureOfRemittance(ddBean[35]);
		ddRecord.setAckNoForm15CA(ddBean[36]);
		ddRecord.setRemittanceCountry(ddBean[37]);
		
		ddRecord.setDeducteeHash(ddBean[38]);
		
		return ddRecord;
	}
	
}
